@@ -67,7 +67,7 @@ struct list_entity {
 
 /**
  * initialize a list-head
- * @param pointer to list-head
+ * @param head to list-head
  */
 static INLINE void
 list_init_head(struct list_entity *head) {
@@ -76,7 +76,7 @@ list_init_head(struct list_entity *head) {
 
 /**
  * initialize a list-node
- * @param pointer to list-node
+ * @param entity to list-node
  */
 static INLINE void
 list_init_node(struct list_entity *entity) {
@@ -129,7 +129,7 @@ list_add_before(struct list_entity *before, struct list_entity *new) {
 
 /**
  * adds a node after another node
- * @param before reference node in the list
+ * @param after reference node in the list
  * @param new node which will be added to the list
  */
 static INLINE void
@@ -164,7 +164,7 @@ list_remove(struct list_entity *entity) {
  * @return true if list is empty, false otherwise
  */
 static INLINE bool
-list_is_empty(struct list_entity *head) {
+list_is_empty(const struct list_entity *head) {
   return head->next == head && head->prev == head;
 }
 
@@ -175,7 +175,7 @@ list_is_empty(struct list_entity *head) {
  *   false otherwise
  */
 static INLINE bool
-list_is_node_added(struct list_entity *node) {
+list_is_node_added(const struct list_entity *node) {
   return node->next != NULL && node->prev != NULL;
 }
 
@@ -213,10 +213,11 @@ list_merge(struct list_entity *add_to, struct list_entity *remove_from) {
     return;
   }
 
-  add_to->next->prev = remove_from->prev;
-  remove_from->prev->next = add_to->next;
-  add_to->next = remove_from->next;
-  remove_from->next->prev = add_to;
+  add_to->prev->next = remove_from->next;
+  remove_from->next->prev = add_to->prev;
+
+  add_to->prev= remove_from->prev;
+  remove_from->prev->next = add_to;
 
   list_init_head(remove_from);
 }
