@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #include <assert.h>
 #include <errno.h>
 #include <limits.h>
@@ -65,16 +69,20 @@ static bool _validate_cfg_entry(
     struct cfg_named_section *named, struct cfg_entry *entry,
     const char *section_name, bool cleanup, struct autobuf *out);
 static bool _section_needs_default_named_one(struct cfg_section_type *type);
-static void _handle_named_section_change(struct cfg_schema_section *s_section,
-    struct cfg_db *pre_change, struct cfg_db *post_change,
-    const char *name, bool startup,
-    struct cfg_named_section *pre_defnamed,
-    struct cfg_named_section *post_defnamed);
+static void _handle_named_section_change (struct cfg_schema_section* s_section, 
+    struct cfg_db* pre_change, struct cfg_db* post_change, const char* section_name, 
+    bool startup, struct cfg_named_section* pre_defnamed, 
+    struct cfg_named_section* post_defnamed);
 static int _handle_db_changes(struct cfg_db *pre_change,
     struct cfg_db *post_change, bool startup);
 
+/*! string array with boolean options with value true */
 const char *CFGLIST_BOOL_TRUE[] = { CFGLIST_BOOL_TRUE_VALUES };
+
+/*! string arrray with valid boolean options */
 const char *CFGLIST_BOOL[] = { CFGLIST_BOOL_VALUES };
+
+/*! text values for configuration schema modes */
 const char *CFG_SCHEMA_SECTIONMODE[CFG_SSMODE_MAX] = {
   [CFG_SSMODE_UNNAMED] = "unnamed",
   [CFG_SSMODE_NAMED] = "named",
@@ -672,6 +680,10 @@ cfg_schema_tobin_strptr(const struct cfg_schema_entry *s_entry __attribute__((un
     const struct const_strarray *value, void *reference) {
   char **ptr;
 
+  if (s_entry->bin_size != sizeof(*ptr)) {
+    return -1;
+  }
+
   ptr = (char **)reference;
   if (*ptr) {
     free(*ptr);
@@ -694,6 +706,10 @@ cfg_schema_tobin_strarray(const struct cfg_schema_entry *s_entry,
     const struct const_strarray *value, void *reference) {
   char *ptr;
 
+  if (s_entry->bin_size < s_entry->validate_param[0].s) {
+    return -1;
+  }
+
   ptr = (char *)reference;
 
   strscpy(ptr, strarray_get_first_c(value), s_entry->validate_param[0].s);
@@ -714,6 +730,10 @@ cfg_schema_tobin_choice(const struct cfg_schema_entry *s_entry,
     const struct const_strarray *value, void *reference) {
   int *ptr;
 
+  if (s_entry->bin_size != sizeof(int)) {
+    return -1;
+  }
+
   ptr = (int *)reference;
 
   *ptr = cfg_get_choice_index(strarray_get_first_c(value),
@@ -735,10 +755,14 @@ cfg_schema_tobin_int(const struct cfg_schema_entry *s_entry,
   int64_t i;
   int result;
 
+  if (s_entry->bin_size != s_entry->validate_param[2].u16[0]) {
+    return -1;
+  }
+
   result = isonumber_to_s64(&i, strarray_get_first_c(value),
-      s_entry->validate_param[2].i16[1], s_entry->validate_param[2].i16[2] == 2);
+      s_entry->validate_param[2].u16[1], s_entry->validate_param[2].u16[2] == 2);
   if (result == 0) {
-    switch (s_entry->validate_param[2].i16[0]) {
+    switch (s_entry->validate_param[2].u16[0]) {
       case 4:
         *((int32_t *)reference) = i;
         break;
@@ -764,6 +788,10 @@ cfg_schema_tobin_netaddr(const struct cfg_schema_entry *s_entry __attribute__((u
     const struct const_strarray *value, void *reference) {
   struct netaddr *ptr;
 
+  if (s_entry->bin_size != sizeof(*ptr)) {
+    return -1;
+  }
+
   ptr = (struct netaddr *)reference;
 
   return netaddr_from_string(ptr, strarray_get_first_c(value));
@@ -782,6 +810,10 @@ cfg_schema_tobin_acl(const struct cfg_schema_entry *s_entry __attribute__((unuse
      const struct const_strarray *value, void *reference) {
   struct netaddr_acl *ptr;
 
+  if (s_entry->bin_size != sizeof(*ptr)) {
+    return -1;
+  }
+
   ptr = (struct netaddr_acl *)reference;
   netaddr_acl_remove(ptr);
 
@@ -803,6 +835,10 @@ cfg_schema_tobin_bitmap256(const struct cfg_schema_entry *s_entry __attribute__(
   const char *ptr;
   int idx;
 
+  if (s_entry->bin_size != sizeof(*bitmap)) {
+    return -1;
+  }
+
   bitmap = (struct bitmap256 *)reference;
   memset(bitmap, 0, sizeof(*bitmap));
 
@@ -843,6 +879,10 @@ cfg_schema_tobin_bool(const struct cfg_schema_entry *s_entry __attribute__((unus
     const struct const_strarray *value, void *reference) {
   bool *ptr;
 
+  if (s_entry->bin_size != sizeof(*ptr)) {
+    return -1;
+  }
+
   ptr = (bool *)reference;
 
   *ptr = cfg_get_bool(strarray_get_first_c(value));
@@ -862,6 +902,10 @@ cfg_schema_tobin_stringlist(const struct cfg_schema_entry *s_entry __attribute__
     const struct const_strarray *value, void *reference) {
   struct strarray *array;
 
+  if (s_entry->bin_size != sizeof(*array)) {
+    return -1;
+  }
+
   array = (struct strarray *)reference;
 
   if (!value->value[0]) {
@@ -1006,7 +1050,6 @@ _handle_db_changes(struct cfg_db *pre_change, struct cfg_db *post_change, bool s
 
 /**
  * Validates on configuration entry.
- * @param schema_section pointer to schema section
  * @param db pointer to database
  * @param section pointer to database section type
  * @param named pointer to named section
@@ -1075,12 +1118,15 @@ _validate_cfg_entry(struct cfg_db *db, struct cfg_section_type *section,
  * @param s_section schema entry for section
  * @param pre_change pointer to database before changes
  * @param post_change pointer to database after changes
- * @param name
+ * @param section_name name of section, might be NULL for unnamed one
+ * @param startup true comparison against empty database at startup
+ * @param pre_defnamed named section with default name before change
+ * @param post_defnamed named section with default name after change
  */
 static void
 _handle_named_section_change(struct cfg_schema_section *s_section,
     struct cfg_db *pre_change, struct cfg_db *post_change,
-    const char *name, bool startup,
+    const char *section_name, bool startup,
     struct cfg_named_section *pre_defnamed,
     struct cfg_named_section *post_defnamed) {
   struct cfg_schema_entry *entry;
@@ -1090,7 +1136,7 @@ _handle_named_section_change(struct cfg_schema_section *s_section,
   if ((s_section->mode == CFG_SSMODE_NAMED
        || s_section->mode == CFG_SSMODE_NAMED_MANDATORY
        || s_section->mode == CFG_SSMODE_NAMED_WITH_DEFAULT)
-      && name == NULL) {
+      && section_name == NULL) {
     /*
      * ignore unnamed data entry for named sections, they are only
      * used for delivering defaults
@@ -1098,11 +1144,11 @@ _handle_named_section_change(struct cfg_schema_section *s_section,
     return;
   }
 
-  s_section->pre = cfg_db_find_namedsection(pre_change, s_section->type, name);
-  s_section->post = cfg_db_find_namedsection(post_change, s_section->type, name);
+  s_section->pre = cfg_db_find_namedsection(pre_change, s_section->type, section_name );
+  s_section->post = cfg_db_find_namedsection(post_change, s_section->type, section_name );
 
   if (s_section->mode == CFG_SSMODE_NAMED_WITH_DEFAULT
-      && strcasecmp(s_section->def_name, name) == 0) {
+      && strcasecmp(s_section->def_name, section_name ) == 0) {
     /* use the default named sections if necessary */
     if (s_section->pre == NULL && !startup) {
       s_section->pre = pre_defnamed;
@@ -1127,16 +1173,16 @@ _handle_named_section_change(struct cfg_schema_section *s_section,
 
     /* read values */
     entry->pre = cfg_db_get_entry_value(
-        pre_change, s_section->type, name, entry->key.entry);
+        pre_change, s_section->type, section_name, entry->key.entry);
     entry->post = cfg_db_get_entry_value(
-        post_change, s_section->type, name, entry->key.entry);
+        post_change, s_section->type, section_name, entry->key.entry);
 
     entry->delta_changed = strarray_cmp_c(entry->pre, entry->post);
     changed |= entry->delta_changed;
   }
 
   if (changed || startup) {
-    s_section->section_name = name;
+    s_section->section_name = section_name;
     s_section->cb_delta_handler();
   }
 }
