@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #include <string.h>
 
 #include "common/common_types.h"
@@ -213,15 +217,15 @@ static struct rfc5444_reader_tlvblock_consumer_entry _pkt_tlvs[] = {
   [IDX_PKTTLV_SEND] =
     { .type = RFC7182_PKTTLV_TIMESTAMP, .mandatory = true,
       .type_ext = RFC7182_TIMESTAMP_EXT_MONOTONIC, .match_type_ext = true,
-      .min_length = 4, .max_length = 4, .match_length = 4, },
+      .min_length = 4, .max_length = 4, .match_length = true, },
   [IDX_PKTTLV_QUERY] =
     { .type = RFC5444_PKTTLV_CHALLENGE,
       .type_ext = RFC5444_CHALLENGE_QUERY, .match_type_ext = true,
-      .min_length = 4, .max_length = 4, .match_length = 4, },
+      .min_length = 4, .max_length = 4, .match_length = true, },
   [IDX_PKTTLV_RESPONSE] =
     { .type = RFC5444_PKTTLV_CHALLENGE,
       .type_ext = RFC5444_CHALLENGE_RESPONSE, .match_type_ext = true,
-      .min_length = 4, .max_length = 4, .match_length = 4, },
+      .min_length = 4, .max_length = 4, .match_length = true, },
 };
 
 static struct rfc5444_writer_pkthandler _pkt_handler = {
@@ -248,7 +252,7 @@ static struct oonf_timer_class _query_trigger_class = {
 };
 
 /* global "timestamp" for replay protection */
-uint32_t _local_timestamp = 1;
+static uint32_t _local_timestamp = 1;
 
 /**
  * Constructor of subsystem
@@ -396,7 +400,7 @@ _cb_query_trigger(void *ptr) {
 static enum rfc5444_result
 _cb_timestamp_tlv(struct rfc5444_reader_tlvblock_context *context __attribute__((unused))) {
   struct oonf_rfc5444_target *target;
-  struct os_interface *core_if;
+  struct os_interface_listener *core_if;
   struct neighbor_node *node;
   uint32_t timestamp, query, response;
   enum rfc5444_result result;
@@ -407,7 +411,7 @@ _cb_timestamp_tlv(struct rfc5444_reader_tlvblock_context *context __attribute__(
 
   struct neighbor_key key;
 
-  core_if = oonf_rfc5444_get_core_interface(_protocol->input_interface);
+  core_if = oonf_rfc5444_get_core_if_listener(_protocol->input_interface);
 
   /* get input-addr/interface combination */
   memset(&key, 0, sizeof(key));
@@ -533,7 +537,7 @@ _cb_timestamp_failed(struct rfc5444_reader_tlvblock_context *context __attribute
 static void
 _cb_addPacketTLVs(struct rfc5444_writer *writer, struct rfc5444_writer_target *rfc5444_target) {
   struct oonf_rfc5444_target *target;
-  struct os_interface *core_if;
+  struct os_interface_listener *core_if;
   struct neighbor_node *node;
   uint32_t query, response;
 #ifdef OONF_LOG_DEBUG_INFO
@@ -546,7 +550,7 @@ _cb_addPacketTLVs(struct rfc5444_writer *writer, struct rfc5444_writer_target *r
   target = oonf_rfc5444_get_target_from_rfc5444_target(rfc5444_target);
 
   /* get core interface */
-  core_if = oonf_rfc5444_get_core_interface(target->interface);
+  core_if = oonf_rfc5444_get_core_if_listener(target->interface);
 
   /* get input-addr/interface combination */
   memset(&key, 0, sizeof(key));
