@@ -67,8 +67,6 @@
 #include "ff_dat_metric/ff_dat_metric.h"
 
 /* Definitions */
-#define LOG_FF_DAT _olsrv2_ffdat_subsystem.logging
-
 enum {
   DAT_SAMPLING_COUNT = 32,
 };
@@ -77,9 +75,6 @@ enum {
  * Configuration settings of DATFF Metric
  */
 struct ff_dat_if_config {
-  /*! Interval between two updates of the metric */
-  uint64_t interval;
-
   /*! true if metric should include link speed */
   bool ett;
 
@@ -97,20 +92,6 @@ struct ff_dat_if_config {
 
   /*! true if we registered the interface */
   bool registered;
-
-#ifdef COLLECT_RAW_DATA
-  /* filename to store raw data into */
-  char *rawdata_file;
-
-  /* true if metric should collect raw data */
-  bool rawdata_start;
-
-  /* time in milliseconds until measurement stops */
-  uint64_t rawdata_maxtime;
-
-  /* maxmimum number of measured packets until measurement stops */
-  int rawdata_maxpackets;
-#endif
 };
 
 /**
@@ -119,13 +100,13 @@ struct ff_dat_if_config {
  */
 struct link_datff_bucket {
   /*! number of RFC5444 packets received in time interval */
-  int received;
+  uint32_t received;
 
   /*! sum of received and lost RFC5444 packets in time interval */
-  int total;
+  uint32_t total;
 
   /*! link speed scaled to "minimum speed = 1" */
-  int scaled_speed;
+  uint32_t scaled_speed;
 };
 
 /**
@@ -164,6 +145,7 @@ struct link_datff_data {
 };
 
 /* prototypes */
+static void _early_cfg_init(void);
 static int _init(void);
 static void _cleanup(void);
 
@@ -231,8 +213,6 @@ static const char *LOSS_SCALING[] = {
 };
 
 static struct cfg_schema_entry _datff_entries[] = {
-  CFG_MAP_CLOCK_MIN(ff_dat_if_config, interval, "ffdat_interval", "1.0",
-      "Time interval between recalculations of metric", 100),
   CFG_MAP_BOOL(ff_dat_if_config, ett, "ffdat_airtime", "true",
       "Activates the handling of linkspeed within the metric, set to false to"
       " downgrade to ETX metric"),
@@ -242,16 +222,6 @@ static struct cfg_schema_entry _datff_entries[] = {
       "Activates the MIC penalty-factor for link metrics"),
   CFG_MAP_BOOL(ff_dat_if_config, accept_unicast, "ffdat_unicast", "false",
       "Include unicast into metric calculation"),
-#ifdef COLLECT_RAW_DATA
-  CFG_MAP_STRING(ff_dat_config, rawdata_file, "raw_filename", "/tmp/olsrv2_dat_metric.txt",
-      "File to write recorded data into"),
-  CFG_MAP_BOOL(ff_dat_config, rawdata_start, "raw_start", "false",
-      "Set to true to activate rawdata measurement"),
-  CFG_MAP_CLOCK(ff_dat_config, rawdata_maxtime, "raw_maxtime", "3600000",
-      "Time until measurement stops"),
-  CFG_MAP_INT32_MINMAX(ff_dat_config, rawdata_maxpackets, "raw_maxpackets", "20000",
-      "Maximum number of packets to record", 0, false, 1, INT32_MAX),
-#endif
 };
 
 /* Subsystem definition */
@@ -271,19 +241,20 @@ static const char *_dependencies[] = {
   OONF_NHDP_SUBSYSTEM,
   OONF_OS_INTERFACE_SUBSYSTEM,
 };
-static struct oonf_subsystem _olsrv2_ffdat_subsystem = {
+static struct oonf_subsystem _nhdp_ffdat_subsystem = {
   .name = OONF_FF_DAT_METRIC_SUBSYSTEM,
   .dependencies = _dependencies,
   .dependencies_count = ARRAYSIZE(_dependencies),
-  .descr = "OLSRv2 Funkfeuer Directional Airtime Metric plugin",
+  .descr = "NHDP Funkfeuer Directional Airtime Metric plugin",
   .author = "Henning Rogge",
 
   .cfg_section = &_datff_section,
 
+  .early_cfg_init = _early_cfg_init,
   .init = _init,
   .cleanup = _cleanup,
 };
-DECLARE_OONF_PLUGIN(_olsrv2_ffdat_subsystem);
+DECLARE_OONF_PLUGIN(_nhdp_ffdat_subsystem);
 
 /* RFC5444 packet listener */
 static struct oonf_rfc5444_protocol *_protocol;
@@ -318,7 +289,6 @@ static struct oonf_class_extension _nhdpif_extenstion = {
 static struct oonf_timer_class _sampling_timer_info = {
   .name = "Sampling timer for DATFF-metric",
   .callback = _cb_dat_sampling,
-  .periodic = true,
 };
 
 /* timer class to measure interval between Hellos */
@@ -345,13 +315,19 @@ static struct nhdp_domain_metric _datff_handler = {
 /* Temporary buffer to sort incoming link speed for median calculation */
 static int _rx_sort_array[DAT_SAMPLING_COUNT] = { 0 };
 
-/* rawdata collection */
-#ifdef COLLECT_RAW_DATA
-static struct autobuf _rawdata_buf;
-static int _rawdata_fd = -1;
-static uint64_t _rawdata_end = 0;
-static int _rawdata_count = 0;
-#endif
+/* ff_dat has multiple logging targets */
+enum oonf_log_source LOG_FF_DAT;
+enum oonf_log_source LOG_FF_DAT_RAW;
+
+/**
+ * Initialize additional logging sources for ffdat
+ */
+static void
+_early_cfg_init(void) {
+  LOG_FF_DAT = _nhdp_ffdat_subsystem.logging;
+  LOG_FF_DAT_RAW = oonf_log_register_source(OONF_FF_DAT_METRIC_SUBSYSTEM"_raw");
+}
+
 
 /**
  * Initialize plugin
@@ -378,9 +354,6 @@ _init(void) {
   _protocol = oonf_rfc5444_get_default_protocol();
 
   oonf_rfc5444_add_protocol_pktseqno(_protocol);
-#ifdef COLLECT_RAW_DATA
-  abuf_init(&_rawdata_buf);
-#endif
   return 0;
 }
 
@@ -392,14 +365,6 @@ _cleanup(void) {
   struct nhdp_interface *nhdp_if, *nhdp_if_it;
   struct ff_dat_if_config *ifconfig;
 
-#ifdef COLLECT_RAW_DATA
-  if (_rawdata_fd != -1) {
-    fsync(_rawdata_fd);
-    close(_rawdata_fd);
-  }
-  abuf_free(&_rawdata_buf);
-  free(_datff_config.rawdata_file);
-#endif
   avl_for_each_element_safe(nhdp_interface_get_tree(), nhdp_if, _node, nhdp_if_it) {
     ifconfig = oonf_class_get_extension(&_nhdpif_extenstion, nhdp_if);
     if (ifconfig->registered) {
@@ -660,6 +625,7 @@ _cb_dat_sampling(struct oonf_timer_instance *ptr) {
   struct rfc7181_metric_field encoded_metric;
   struct ff_dat_if_config *ifconfig;
   struct link_datff_data *ldata;
+  struct nhdp_interface *nhdp_if;
   struct nhdp_link *lnk;
   uint32_t total, received;
   uint64_t metric;
@@ -667,21 +633,14 @@ _cb_dat_sampling(struct oonf_timer_instance *ptr) {
   uint32_t missing_intervals;
   size_t i;
   int rx_bitrate;
-  bool change_happened;
-
   struct netaddr_str nbuf;
 
   ifconfig = container_of(ptr, struct ff_dat_if_config, _sampling_timer);
 
   OONF_DEBUG(LOG_FF_DAT, "Calculate Metric from sampled data");
 
-  change_happened = false;
-
-  list_for_each_element(nhdp_db_get_link_list(), lnk, _global_node) {
-    if (oonf_class_get_extension(&_nhdpif_extenstion, lnk->local_if) != ifconfig) {
-      continue;
-    }
-
+  nhdp_if = oonf_class_get_base(&_nhdpif_extenstion, ifconfig);
+  list_for_each_element(&nhdp_if->_links, lnk, _if_node) {
     ldata = oonf_class_get_extension(&_link_extenstion, lnk);
     if (!ldata->contains_data) {
       /* still no data for this link */
@@ -700,7 +659,7 @@ _cb_dat_sampling(struct oonf_timer_instance *ptr) {
 
     if (ldata->missed_hellos > 0) {
       missing_intervals = (ldata->missed_hellos * ldata->hello_interval)
-          / ifconfig->interval;
+          / lnk->local_if->refresh_interval;
       if (missing_intervals > ARRAYSIZE(ldata->buckets)) {
         received = 0;
       }
@@ -712,18 +671,6 @@ _cb_dat_sampling(struct oonf_timer_instance *ptr) {
 
     /* update link speed */
     ldata->buckets[ldata->activePtr].scaled_speed = _get_scaled_rx_linkspeed(ifconfig, lnk);
-#ifdef COLLECT_RAW_DATA
-    if (_rawdata_fd != -1) {
-      if (0 > write(_rawdata_fd, abuf_getptr(&_rawdata_buf), abuf_getlen(&_rawdata_buf))) {
-        close (_rawdata_fd);
-        _rawdata_fd = -1;
-      }
-      else {
-        fsync(_rawdata_fd);
-        abuf_clear(&_rawdata_buf);
-      }
-    }
-#endif
 
     OONF_DEBUG(LOG_FF_DAT, "Query incoming linkspeed for link %s: %"PRIu64,
         netaddr_to_string(&nbuf, &lnk->if_addr),
@@ -776,7 +723,7 @@ _cb_dat_sampling(struct oonf_timer_instance *ptr) {
     }
 
     /* set metric for incoming link */
-    change_happened |= nhdp_domain_set_incoming_metric(
+    nhdp_domain_set_incoming_metric(
         &_datff_handler, lnk, metric_value);
 
     OONF_DEBUG(LOG_FF_DAT, "New sampling rate for link %s (%s):"
@@ -793,11 +740,7 @@ _cb_dat_sampling(struct oonf_timer_instance *ptr) {
     ldata->buckets[ldata->activePtr].received = 0;
     ldata->buckets[ldata->activePtr].total = 0;
   }
-
-  /* update neighbor metrics */
-  if (change_happened) {
-    nhdp_domain_neighborhood_changed();
-  }
+oonf_timer_set(&ifconfig->_sampling_timer, nhdp_if->refresh_interval);
 }
 
 /**
@@ -961,9 +904,8 @@ _shall_process_packet(struct nhdp_interface *nhdpif, struct ff_dat_if_config *if
 /**
  * Callback to process all in RFC5444 packets for metric calculation. The
  * Callback ignores all unicast packets.
- * @param consumer
- * @param context
- * @return
+ * @param context RFC5444 context of the incoming packet
+ * @return RFC5444 API result
  */
 static enum rfc5444_result
 _cb_process_packet(struct rfc5444_reader_tlvblock_context *context) {
@@ -974,7 +916,10 @@ _cb_process_packet(struct rfc5444_reader_tlvblock_context *context) {
   struct nhdp_link *lnk;
   int total;
 
+#ifdef OONF_LOG_DEBUG_INFO
   struct netaddr_str nbuf;
+  struct isonumber_str timebuf;
+#endif
 
   if (!context->has_pktseqno) {
     struct netaddr_str buf;
@@ -1003,45 +948,17 @@ _cb_process_packet(struct rfc5444_reader_tlvblock_context *context) {
     return RFC5444_OKAY;
   }
 
-#ifdef COLLECT_RAW_DATA
-  if (_rawdata_fd != -1)
-  {
-    uint64_t now;
-
-    now = oonf_clock_getNow();
-    _rawdata_count++;
-
-    if (now > _rawdata_end || _rawdata_count > _datff_config.rawdata_maxpackets) {
-      if (0 <= write(_rawdata_fd, abuf_getptr(&_rawdata_buf), abuf_getlen(&_rawdata_buf))) {
-        fsync(_rawdata_fd);
-      }
-      close(_rawdata_fd);
-      _rawdata_fd = -1;
-    }
-    else {
-      struct isonumber_str timebuf;
-      struct netaddr_str neighbuf;
-
-      abuf_appendf(&_rawdata_buf, "%s %s %u %d\n",
-          oonf_clock_toIntervalString(&timebuf, now),
-          netaddr_to_string(&neighbuf, &laddr->link_addr),
+  /* log raw metric data */
+  OONF_DEBUG(LOG_FF_DAT_RAW, "%s %s %u %d\n",
+          oonf_clock_toIntervalString(&timebuf, oonf_clock_getNow()),
+          netaddr_to_string(&nbuf, &laddr->link_addr),
           context->pkt_seqno,
-          _get_scaled_rx_linkspeed(laddr->link));
-    }
-  }
-#endif
+          _get_scaled_rx_linkspeed(ifconfig, laddr->link));
 
   /* get link and its dat data */
   lnk = laddr->link;
   ldata = oonf_class_get_extension(&_link_extenstion, lnk);
 
-  if (!ldata->buckets) {
-    OONF_WARN(LOG_FF_DAT, "No buckets for link to %s (%s)",
-        netaddr_to_string(&nbuf, &lnk->if_addr),
-        nhdp_interface_get_name(lnk->local_if));
-    return RFC5444_OKAY;
-  }
-
   if (!ldata->contains_data) {
     ldata->contains_data = true;
     ldata->activePtr = 0;
@@ -1052,9 +969,11 @@ _cb_process_packet(struct rfc5444_reader_tlvblock_context *context) {
     return RFC5444_OKAY;
   }
 
-  total = (int)(context->pkt_seqno) - (int)(ldata->last_seq_nr);
-  if (total < 0) {
-    total += 65536;
+  if (context->pkt_seqno >= ldata->last_seq_nr) {
+    total = context->pkt_seqno - ldata->last_seq_nr;
+  }
+  else {
+    total = ((uint32_t)(context->pkt_seqno) + 65536) - (uint32_t)(ldata->last_seq_nr);
   }
 
   ldata->buckets[ldata->activePtr].received++;
@@ -1134,17 +1053,8 @@ _int_link_to_string(struct nhdp_metric_str *buf, struct nhdp_link *lnk) {
   int64_t received = 0, total = 0;
   size_t i;
 
-  struct netaddr_str nbuf;
-
   ldata = oonf_class_get_extension(&_link_extenstion, lnk);
 
-  if (!ldata->buckets) {
-    OONF_WARN(LOG_FF_DAT, "No buckets for link to %s (%s)",
-        netaddr_to_string(&nbuf, &lnk->if_addr),
-        nhdp_interface_get_name(lnk->local_if));
-    return RFC5444_OKAY;
-  }
-
   for (i=0; i<ARRAYSIZE(ldata->buckets); i++) {
     received += ldata->buckets[i].received;
     total += ldata->buckets[i].total;
@@ -1209,24 +1119,5 @@ _cb_cfg_changed(void) {
   }
 
   /* start/change sampling timer */
-  oonf_timer_set(&ifconfig->_sampling_timer, ifconfig->interval);
-
-#ifdef COLLECT_RAW_DATA
-  if (_rawdata_fd != -1) {
-    fsync(_rawdata_fd);
-    close(_rawdata_fd);
-    _rawdata_end = 0;
-    _rawdata_count = 0;
-  }
-
-  if (_datff_config.rawdata_start) {
-    _rawdata_fd = open(_datff_config.rawdata_file, O_CREAT | O_TRUNC | O_WRONLY,
-    		S_IRUSR|S_IWUSR);
-    if (_rawdata_fd != -1) {
-      abuf_clear(&_rawdata_buf);
-      abuf_appendf(&_rawdata_buf, "Time: %s\n", oonf_log_get_walltime());
-      _rawdata_end = oonf_clock_get_absolute(_datff_config.rawdata_maxtime);
-    }
-  }
-#endif
+  oonf_timer_set(&ifconfig->_sampling_timer, 1000);
 }
