@@ -139,6 +139,9 @@ static struct avl_tree _neigh_originator_tree;
 /* list of links (to neighbors) */
 static struct list_entity _link_list;
 
+/* id that will be increased every times the symmetric neighbor set changes */
+static uint32_t _neighbor_set_id = 0;
+
 /**
  * Initialize NHDP databases
  */
@@ -229,11 +232,14 @@ nhdp_db_neighbor_add(void) {
  */
 void
 nhdp_db_neighbor_remove(struct nhdp_neighbor *neigh) {
+  struct nhdp_neighbor_domaindata *neighdata;
   struct nhdp_naddr *naddr, *na_it;
   struct nhdp_link *lnk, *l_it;
+  struct nhdp_domain *domain;
 #ifdef OONF_LOG_DEBUG_INFO
   struct netaddr_str nbuf;
 #endif
+  bool was_mpr;
 
   OONF_DEBUG(LOG_NHDP, "Remove Neighbor: 0x%0zx (%s)",
       (size_t)neigh, netaddr_to_string(&nbuf, &neigh->originator));
@@ -259,6 +265,21 @@ nhdp_db_neighbor_remove(struct nhdp_neighbor *neigh) {
     avl_remove(&_neigh_originator_tree, &neigh->_originator_node);
   }
 
+  /* check if neighbor was a MPR */
+  was_mpr = false;
+  list_for_each_element(nhdp_domain_get_list(), domain, _node) {
+    neighdata = nhdp_domain_get_neighbordata(domain, neigh);
+    if (neighdata->neigh_is_mpr) {
+      was_mpr = true;
+      break;
+    }
+  }
+
+  if (was_mpr) {
+    /* all domains might have changed */
+    nhdp_domain_delayed_mpr_recalculation(NULL, neigh);
+  }
+
   /* remove from global list and free memory */
   list_remove(&neigh->_global_node);
   oonf_class_free(&_neigh_info, neigh);
@@ -391,8 +412,8 @@ nhdp_db_neighbor_addr_remove(struct nhdp_naddr *naddr) {
 
 /**
  * Moves a nhdp neighbor address to a different neighbor
- * @param neigh
- * @param naddr
+ * @param neigh nhdp neighbor
+ * @param naddr nhdp neighbor address
  */
 void
 nhdp_db_neighbor_addr_move(struct nhdp_neighbor *neigh, struct nhdp_naddr *naddr) {
@@ -457,6 +478,12 @@ nhdp_db_neighbor_set_originator(struct nhdp_neighbor *neigh,
       avl_insert(&lnk->local_if->_link_originators, &lnk->_originator_node);
     }
   }
+
+  /* inform everyone */
+  oonf_class_event(&_neigh_info, neigh, OONF_OBJECT_CHANGED);
+
+  /* overwrite "old originator" */
+  memcpy(&neigh->_old_originator, originator, sizeof(*originator));
 }
 
 /**
@@ -491,6 +518,15 @@ nhdp_db_neigbor_disconnect_dualstack(struct nhdp_neighbor *neigh) {
   }
 }
 
+/**
+ * @return set id of symmetric neighbors, will be increased for
+ *   every change.
+ */
+uint32_t
+nhdp_db_neighbor_get_set_id(void) {
+  return _neighbor_set_id;
+}
+
 /**
  * Insert a new link into a nhdp neighbors database
  * @param neigh neighbor which will get the new link
@@ -670,8 +706,8 @@ nhdp_db_link_addr_remove(struct nhdp_laddr *laddr) {
 
 /**
  * Moves a nhdp link address to a different link
- * @param lnk
- * @param laddr
+ * @param lnk NHDP link 
+ * @param laddr NHDP link address
  */
 void
 nhdp_db_link_addr_move(struct nhdp_link *lnk, struct nhdp_laddr *laddr) {
@@ -800,6 +836,7 @@ nhdp_db_link_update_status(struct nhdp_link *lnk) {
   was_symmetric = lnk->status == NHDP_LINK_SYMMETRIC;
 
   /* update link status */
+  lnk->last_status = lnk->status;
   lnk->status = _nhdp_db_link_calculate_status(lnk);
 
   /* handle database changes */
@@ -818,10 +855,12 @@ nhdp_db_link_update_status(struct nhdp_link *lnk) {
   if (old_status != lnk->status) {
     /* link status was changed */
     lnk->last_status_change = oonf_clock_getNow();
-  }
+    nhdp_domain_recalculate_metrics(NULL, lnk->neigh);
+    nhdp_domain_delayed_mpr_recalculation(NULL, lnk->neigh);
 
-  /* trigger change event */
-  oonf_class_event(&_link_info, lnk, OONF_OBJECT_CHANGED);
+    /* trigger change event */
+    oonf_class_event(&_link_info, lnk, OONF_OBJECT_CHANGED);
+  }
 }
 
 /**
@@ -885,6 +924,32 @@ nhdp_db_get_neigh_originator_tree(void) {
  */
 int
 _nhdp_db_link_calculate_status(struct nhdp_link *lnk) {
+  const struct netaddr *local_originator;
+  const struct os_interface *localif;
+  int af_type;
+
+  af_type = netaddr_get_address_family(&lnk->if_addr);
+  local_originator = nhdp_get_originator(af_type);
+  localif = nhdp_interface_get_if_listener(lnk->local_if)->data;
+
+  /* check for originator collision */
+  if (!netaddr_is_unspec(local_originator)
+      && (netaddr_cmp(local_originator, &lnk->if_addr) == 0
+          || netaddr_cmp(local_originator, &lnk->neigh->originator) == 0)) {
+    return NHDP_LINK_PENDING;
+  }
+
+  /* check for interface address collision */
+  if (nhdp_interface_addr_if_get(lnk->local_if, &lnk->if_addr)) {
+    return NHDP_LINK_PENDING;
+  }
+
+  /* check for MAC collision */
+  if (netaddr_cmp(&localif->mac, &lnk->remote_mac) == 0) {
+    return NHDP_LINK_PENDING;
+  }
+
+  /* calculate link status as described in RFC 6130 */
   if (nhdp_hysteresis_is_pending(lnk))
     return NHDP_LINK_PENDING;
   if (nhdp_hysteresis_is_lost(lnk))
@@ -910,6 +975,7 @@ _link_status_now_symmetric(struct nhdp_link *lnk) {
     avl_for_each_element(&lnk->neigh->_neigh_addresses, naddr, _neigh_node) {
       nhdp_db_neighbor_addr_not_lost(naddr);
     }
+    _neighbor_set_id++;
   }
 }
 
@@ -933,6 +999,8 @@ _link_status_not_symmetric_anymore(struct nhdp_link *lnk) {
     avl_for_each_element_safe(&lnk->neigh->_neigh_addresses, naddr, _neigh_node, na_it) {
       nhdp_db_neighbor_addr_set_lost(naddr, lnk->local_if->n_hold_time);
     }
+
+    _neighbor_set_id++;
   }
 }
 
@@ -960,10 +1028,6 @@ _cb_link_vtime(struct oonf_timer_instance *ptr) {
   /* check if neighbor still has links */
   if (list_is_empty(&neigh->_links)) {
     nhdp_db_neighbor_remove(neigh);
-    nhdp_domain_neighborhood_changed();
-  }
-  else {
-    nhdp_domain_neighbor_changed(neigh);
   }
 }
 
@@ -991,7 +1055,6 @@ _cb_link_symtime(struct oonf_timer_instance *ptr) {
   lnk = container_of(ptr, struct nhdp_link, sym_time);
   OONF_DEBUG(LOG_NHDP, "Link Symtime fired: 0x%0zx", (size_t)lnk);
   nhdp_db_link_update_status(lnk);
-  nhdp_domain_neighbor_changed(lnk->neigh);
 }
 
 /**
@@ -1015,12 +1078,9 @@ _cb_naddr_vtime(struct oonf_timer_instance *ptr) {
 static void
 _cb_l2hop_vtime(struct oonf_timer_instance *ptr) {
   struct nhdp_l2hop *l2hop;
-  struct nhdp_neighbor *neigh;
 
   l2hop = container_of(ptr, struct nhdp_l2hop, _vtime);
-  neigh = l2hop->link->neigh;
 
   OONF_DEBUG(LOG_NHDP, "2Hop vtime fired: 0x%0zx", (size_t)ptr);
   nhdp_db_link_2hop_remove(l2hop);
-  nhdp_domain_neighbor_changed(neigh);
 }
