@@ -66,7 +66,7 @@
 
 /* Prototypes of local functions */
 static void _addr_add(struct nhdp_interface *, struct netaddr *addr);
-static void _addr_remove(struct nhdp_interface_addr *addr, uint64_t vtime);
+static void _addr_has_been_removed(struct nhdp_interface_addr *addr, uint64_t vtime);
 static void _remove_addr(struct nhdp_interface_addr *ptr);
 static void _cb_addr_timeout(struct oonf_timer_instance *ptr);
 
@@ -129,7 +129,9 @@ nhdp_interfaces_cleanup(void) {
   struct nhdp_interface *interf, *if_it;
 
   avl_for_each_element_safe(&_interface_tree, interf, _node, if_it) {
-    nhdp_interface_remove(interf);
+    if (interf->registered) {
+      nhdp_interface_remove(interf);
+    }
   }
 
   oonf_timer_remove(&_interface_hello_timer);
@@ -146,7 +148,7 @@ nhdp_interfaces_cleanup(void) {
 void
 nhdp_interface_update_status(struct nhdp_interface *interf) {
   struct nhdp_link *lnk;
-  int ipv4_only, ipv6_only, dualstack;
+  uint32_t ipv4_only, ipv6_only, dualstack;
 
   ipv4_only = 0;
   ipv6_only = 0;
@@ -355,7 +357,7 @@ nhdp_interface_get_address_tree(void) {
 /**
  * Add a nhdp interface address to an interface
  * @param interf pointer to nhdp interface
- * @param if_addr address to add to interface
+ * @param addr address to add to interface
  */
 void
 _addr_add(struct nhdp_interface *interf, struct netaddr *addr) {
@@ -405,7 +407,7 @@ _addr_add(struct nhdp_interface *interf, struct netaddr *addr) {
  * @param vtime time in milliseconds until address should be removed from db
  */
 static void
-_addr_remove(struct nhdp_interface_addr *addr, uint64_t vtime) {
+_addr_has_been_removed(struct nhdp_interface_addr *addr, uint64_t vtime) {
 #ifdef OONF_LOG_DEBUG_INFO
   struct netaddr_str buf;
 #endif
@@ -419,7 +421,7 @@ _addr_remove(struct nhdp_interface_addr *addr, uint64_t vtime) {
 
 /**
  * remove address from NHDP interface
- * @param ptr pointer to nhdp interface address
+ * @param addr nhdp interface address
  */
 static void
 _remove_addr(struct nhdp_interface_addr *addr) {
@@ -448,7 +450,6 @@ _cb_addr_timeout(struct oonf_timer_instance *ptr) {
  * AVL tree comparator for netaddr objects.
  * @param k1 pointer to key 1
  * @param k2 pointer to key 2
- * @param ptr not used in this comparator
  * @return +1 if k1>k2, -1 if k1<k2, 0 if k1==k2
  */
 static int
@@ -543,7 +544,7 @@ _cb_interface_event(struct oonf_rfc5444_interface_listener *ifl,
   avl_for_each_element_safe(&interf->_if_addresses, addr, _if_node, addr_it) {
     if (addr->_to_be_removed && !addr->removed) {
       addr->_to_be_removed = false;
-      _addr_remove(addr, interf->i_hold_time);
+      _addr_has_been_removed(addr, interf->i_hold_time);
     }
   }
 
