@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #include "common/common_types.h"
 #include "common/avl.h"
 #include "common/avl_comp.h"
@@ -81,7 +85,7 @@ static void _write_metric_tlv(struct rfc5444_writer *writer,
 static struct rfc5444_writer_message *_nhdp_message = NULL;
 
 static struct rfc5444_writer_content_provider _nhdp_msgcontent_provider = {
-  .msg_type = RFC5444_MSGTYPE_HELLO,
+  .msg_type = RFC6130_MSGTYPE_HELLO,
   .addMessageTLVs = _cb_addMessageTLVs,
   .addAddresses = _cb_addAddresses,
 };
@@ -96,17 +100,20 @@ static struct rfc5444_writer_tlvtype _nhdp_addrtlvs[] = {
 static struct oonf_rfc5444_protocol *_protocol;
 
 static bool _cleanedup = false;
+static bool _add_mac_tlv = true;
 static struct nhdp_interface *_nhdp_if = NULL;
 
 /**
  * Initialize nhdp writer
+ * @param p rfc5444 protocol
+ * @return -1 if an error happened, 0 otherwise
  */
 int
 nhdp_writer_init(struct oonf_rfc5444_protocol *p) {
   _protocol = p;
 
   _nhdp_message = rfc5444_writer_register_message(
-      &_protocol->writer, RFC5444_MSGTYPE_HELLO, true);
+      &_protocol->writer, RFC6130_MSGTYPE_HELLO, true);
   if (_nhdp_message == NULL) {
     OONF_WARN(LOG_NHDP_W, "Could not register NHDP Hello message");
     return -1;
@@ -143,12 +150,12 @@ nhdp_writer_cleanup(void) {
 /**
  * Send a NHDP Hello through the specified interface. This might result
  * in both an IPv4 and IPv6 message
- * @param interf NHDP interface
+ * @param ninterf NHDP interface
  */
 void
 nhdp_writer_send_hello(struct nhdp_interface *ninterf) {
   enum rfc5444_result result;
-  struct os_interface *interf;
+  struct os_interface_listener *interf;
   struct netaddr_str buf;
 
   if (_cleanedup) {
@@ -156,8 +163,8 @@ nhdp_writer_send_hello(struct nhdp_interface *ninterf) {
     return;
   }
 
-  interf = nhdp_interface_get_coreif(ninterf);
-  if (interf->data.loopback) {
+  interf = nhdp_interface_get_if_listener(ninterf);
+  if (interf->data->flags.loopback) {
     /* no NHDP on loopback interface */
     return;
   }
@@ -165,28 +172,40 @@ nhdp_writer_send_hello(struct nhdp_interface *ninterf) {
   OONF_DEBUG(LOG_NHDP_W, "Sending Hello to interface %s",
       nhdp_interface_get_name(ninterf));
 
+  nhdp_domain_recalculate_mpr();
+
   /* store NHDP interface */
   _nhdp_if = ninterf;
 
   /* send IPv4 (if socket is active) */
-  result = oonf_rfc5444_send_if(ninterf->rfc5444_if.interface->multicast4, RFC5444_MSGTYPE_HELLO);
+  result = oonf_rfc5444_send_if(ninterf->rfc5444_if.interface->multicast4, RFC6130_MSGTYPE_HELLO);
   if (result < 0) {
     OONF_WARN(LOG_NHDP_W, "Could not send NHDP message to %s: %s (%d)",
         netaddr_to_string(&buf, &ninterf->rfc5444_if.interface->multicast4->dst), rfc5444_strerror(result), result);
   }
 
   /* send IPV6 (if socket is active) */
-  result = oonf_rfc5444_send_if(ninterf->rfc5444_if.interface->multicast6, RFC5444_MSGTYPE_HELLO);
+  result = oonf_rfc5444_send_if(ninterf->rfc5444_if.interface->multicast6, RFC6130_MSGTYPE_HELLO);
   if (result < 0) {
     OONF_WARN(LOG_NHDP_W, "Could not send NHDP message to %s: %s (%d)",
         netaddr_to_string(&buf, &ninterf->rfc5444_if.interface->multicast6->dst), rfc5444_strerror(result), result);
   }
 }
 
+
+/**
+ * activates or deactivates the MAC_TLV in the NHDP Hello messages
+ * @param active true if MAC_TLV should be present
+ */
+void
+nhdp_writer_set_mac_TLV_state(bool active) {
+  _add_mac_tlv = active;
+}
+
 /**
  * Callback to initialize the message header for a HELLO message
- * @param writer
- * @param message
+ * @param writer RFC5444 writer instance
+ * @param message RFC5444 message
  */
 static int
 _cb_addMessageHeader(struct rfc5444_writer *writer,
@@ -233,15 +252,14 @@ _cb_addMessageHeader(struct rfc5444_writer *writer,
 
 /**
  * Callback to add the message TLVs to a HELLO message
- * @param writer
- * @param prv
+ * @param writer RFC5444 writer
  */
 static void
 _cb_addMessageTLVs(struct rfc5444_writer *writer) {
   uint8_t vtime_encoded, itime_encoded;
   struct oonf_rfc5444_target *target;
   const struct netaddr *v4_originator;
-  struct os_interface_data *ifdata;
+  struct os_interface *os_if;
   uint8_t willingness[NHDP_MAXIMUM_DOMAINS];
   size_t willingness_size;
   uint8_t mprtypes[NHDP_MAXIMUM_DOMAINS];
@@ -291,20 +309,20 @@ _cb_addMessageTLVs(struct rfc5444_writer *writer) {
   }
 
   /* add mac address of local interface */
-  ifdata = oonf_interface_get_data(target->interface->name, NULL);
-  if (ifdata == NULL) {
-    return;
+  os_if = nhdp_interface_get_if_listener(_nhdp_if)->data;
+
+  if (_add_mac_tlv && netaddr_get_address_family(&os_if->mac) == AF_MAC48) {
+    rfc5444_writer_add_messagetlv(writer, NHDP_MSGTLV_MAC, 0,
+        netaddr_get_binptr(&os_if->mac), netaddr_get_binlength(&os_if->mac));
   }
-  rfc5444_writer_add_messagetlv(writer, NHDP_MSGTLV_MAC, 0,
-      netaddr_get_binptr(&ifdata->mac), netaddr_get_binlength(&ifdata->mac));
 }
 
 /**
  * Add a rfc5444 address with localif TLV to the stream
- * @param writer
- * @param prv
- * @param interf
- * @param addr
+ * @param writer RFC5444 writer instance
+ * @param prv RFC5444 content provider instance
+ * @param interf NHDP interface
+ * @param addr NHDP interface address
  */
 static void
 _add_localif_address(struct rfc5444_writer *writer, struct rfc5444_writer_content_provider *prv,
@@ -342,10 +360,10 @@ _add_localif_address(struct rfc5444_writer *writer, struct rfc5444_writer_conten
 
 /**
  * Add a rfc5444 address with link_status or other_neigh TLV to the stream
- * @param writer
- * @param prv
- * @param interf
- * @param naddr
+ * @param writer RFC5444 writer instance
+ * @param prv RFC5444 content provider instance
+ * @param interf NHDP interface
+ * @param naddr NHDP interface address
  */
 static void
 _add_link_address(struct rfc5444_writer *writer, struct rfc5444_writer_content_provider *prv,
@@ -401,7 +419,7 @@ _add_link_address(struct rfc5444_writer *writer, struct rfc5444_writer_content_p
 
   /* add MPR tlvs */
   if (laddr != NULL) {
-    len = nhdp_domain_encode_mpr_tlvvalue(mprvalue, sizeof(mprvalue), naddr->neigh);
+    len = nhdp_domain_encode_mpr_tlvvalue(mprvalue, sizeof(mprvalue), laddr->link);
 
     if (len) {
       rfc5444_writer_add_addrtlv(writer, address,
@@ -430,11 +448,11 @@ _add_link_address(struct rfc5444_writer *writer, struct rfc5444_writer_content_p
 
 /**
  * Write up to four metric TLVs to an address
- * @param writer rfc5444 writer
+ * @param writer rfc5444 writer instance
  * @param addr rfc5444 address
- * @param neigh pointer to symmetric neighbor, might be NULL
- * @param lnk pointer to symmetric link, might be NULL
- * @param handler pointer to link metric handler
+ * @param neigh symmetric NHDP neighbor, might be NULL
+ * @param lnk symmetric NHDP link, might be NULL
+ * @param domain NHDP domain
  */
 static void
 _write_metric_tlv(struct rfc5444_writer *writer, struct rfc5444_writer_address *addr,
@@ -446,6 +464,11 @@ _write_metric_tlv(struct rfc5444_writer *writer, struct rfc5444_writer_address *
       RFC7181_LINKMETRIC_INCOMING_NEIGH,
       RFC7181_LINKMETRIC_OUTGOING_NEIGH,
   };
+#ifdef OONF_LOG_DEBUG_INFO
+  static const char *lq_name[4] = {
+    "l_in", "l_out", "n_in", "n_out",
+  };
+#endif
   struct nhdp_link_domaindata *linkdata;
   struct nhdp_neighbor_domaindata *neighdata;
   struct rfc7181_metric_field metric_encoded[4], tlv_value;
@@ -503,17 +526,20 @@ _write_metric_tlv(struct rfc5444_writer *writer, struct rfc5444_writer_address *
     rfc7181_metric_set_flag(&tlv_value, flags[i]);
 
     /* mark all metric pair that have the same linkmetric */
+    OONF_DEBUG(LOG_NHDP_W, "Add Metric %s (ext %u): 0x%02x%02x (%u)",
+        lq_name[i], domain->ext, tlv_value.b[0], tlv_value.b[1], metrics[i]);
+
     for (j=3; j>i; j--) {
       if (metrics[j] > 0 &&
           memcmp(&metric_encoded[i], &metric_encoded[j], sizeof(metric_encoded[0])) == 0) {
         rfc7181_metric_set_flag(&tlv_value, flags[j]);
         metrics[j] = 0;
+
+        OONF_DEBUG(LOG_NHDP_W, "Same metrics for %s (ext %u)",
+            lq_name[j], domain->ext);
       }
     }
 
-    OONF_DEBUG(LOG_NHDP_W, "Add Metric (ext %u): 0x%02x%02x (%u)",
-        domain->ext, tlv_value.b[0], tlv_value.b[1], metrics[i]);
-
     /* add to rfc5444 address */
     rfc5444_writer_add_addrtlv(writer, addr,
         &domain->_metric_addrtlvs[k++],
@@ -522,7 +548,7 @@ _write_metric_tlv(struct rfc5444_writer *writer, struct rfc5444_writer_address *
 }
 /**
  * Callback to add the addresses and address TLVs to a HELLO message
- * @param writer
+ * @param writer RFC5444 writer instance
  */
 void
 _cb_addAddresses(struct rfc5444_writer *writer) {
