@@ -46,12 +46,13 @@
 #ifndef OLSRV2_INTERNAL_H_
 #define OLSRV2_INTERNAL_H_
 
+#include "common/common_types.h"
 #include "core/oonf_logging.h"
 
-/* headers only for use inside the NHDP subsystem */
-enum oonf_log_source LOG_OLSRV2;
-enum oonf_log_source LOG_OLSRV2_R;
-enum oonf_log_source LOG_OLSRV2_ROUTING;
-enum oonf_log_source LOG_OLSRV2_W;
+/* headers only for use inside the OLSRv2 subsystem */
+EXPORT enum oonf_log_source LOG_OLSRV2;
+EXPORT enum oonf_log_source LOG_OLSRV2_R;
+EXPORT enum oonf_log_source LOG_OLSRV2_ROUTING;
+EXPORT enum oonf_log_source LOG_OLSRV2_W;
 
 #endif /* OLSRV2_INTERNAL_H_ */
