@@ -51,6 +51,7 @@
 #include "common/list.h"
 #include "common/netaddr.h"
 #include "core/oonf_logging.h"
+#include "core/os_core.h"
 #include "subsystems/oonf_class.h"
 #include "subsystems/oonf_rfc5444.h"
 #include "subsystems/oonf_timer.h"
@@ -87,8 +88,11 @@ static void _handle_nhdp_routes(struct nhdp_domain *);
 static void _add_route_to_kernel_queue(struct olsrv2_routing_entry *rtentry);
 static void _process_dijkstra_result(struct nhdp_domain *);
 static void _process_kernel_queue(void);
+
+static void _cb_mpr_update(struct nhdp_domain *);
+static void _cb_metric_update(struct nhdp_domain *);
 static void _cb_trigger_dijkstra(struct oonf_timer_instance *);
-static void _cb_nhdp_update(struct nhdp_neighbor *);
+
 static void _cb_route_finished(struct os_route *route, int error);
 
 /* Domain parameter of dijkstra algorithm */
@@ -110,12 +114,19 @@ static struct oonf_timer_instance _rate_limit_timer = {
   .class = &_dijkstra_timer_info
 };
 
+static bool _trigger_dijkstra = false;
+
 /* callback for NHDP domain events */
 static struct nhdp_domain_listener _nhdp_listener = {
-  .update = _cb_nhdp_update,
+  .mpr_update = _cb_mpr_update,
+  .metric_update = _cb_metric_update,
 };
 
-static bool _trigger_dijkstra = false;
+/* status variables for domain changes */
+static uint16_t _ansn;
+static bool _domain_changed[NHDP_MAXIMUM_DOMAINS];
+static bool _update_ansn;
+
 
 /* global datastructures for routing */
 static struct avl_tree _routing_tree[NHDP_MAXIMUM_DOMAINS];
@@ -125,14 +136,25 @@ static struct avl_tree _dijkstra_working_tree;
 static struct list_entity _kernel_queue;
 
 static bool _initiate_shutdown = false;
+static bool _freeze_routes = false;
 
 /**
  * Initialize olsrv2 dijkstra and routing code
  */
-void
+int
 olsrv2_routing_init(void) {
   int i;
 
+  /* initialize domain change tracker */
+  if (os_core_get_random(&_ansn, sizeof(_ansn))) {
+    return -1;
+  }
+
+  nhdp_domain_listener_add(&_nhdp_listener);
+  memset(_domain_changed, 0, sizeof(_domain_changed));
+  _update_ansn = false;
+
+
   oonf_class_add(&_rtset_entry);
   oonf_timer_add(&_dijkstra_timer_info);
 
@@ -143,7 +165,7 @@ olsrv2_routing_init(void) {
   avl_init(&_dijkstra_working_tree, avl_comp_uint32, true);
   list_init_head(&_kernel_queue);
 
-  nhdp_domain_listener_add(&_nhdp_listener);
+  return 0;
 }
 
 /**
@@ -156,6 +178,7 @@ olsrv2_routing_initiate_shutdown(void) {
 
   /* remember we are in shutdown */
   _initiate_shutdown = true;
+  _freeze_routes = false;
 
   /* remove all routes */
   for (i=0; i<NHDP_MAXIMUM_DOMAINS; i++) {
@@ -185,7 +208,6 @@ olsrv2_routing_cleanup(void) {
   int i;
 
   nhdp_domain_listener_remove(&_nhdp_listener);
-
   oonf_timer_stop(&_rate_limit_timer);
 
   for (i=0; i<NHDP_MAXIMUM_DOMAINS; i++) {
@@ -203,6 +225,23 @@ olsrv2_routing_cleanup(void) {
   oonf_class_remove(&_rtset_entry);
 }
 
+/**
+ * @return current answer set number for local topology database
+ */
+uint16_t
+olsrv2_routing_get_ansn(void) {
+  return _ansn;
+}
+
+/**
+ * Force the answer set number to increase
+ * @param increment amount of increase
+ */
+void
+olsrv2_routing_force_ansn_increment(uint16_t increment) {
+  _ansn += increment;
+}
+
 /**
  * Trigger a new dijkstra as soon as we are back in the mainloop
  * (unless the rate limitation timer is active, then we will wait for it)
@@ -218,6 +257,24 @@ olsrv2_routing_trigger_update(void) {
   OONF_DEBUG(LOG_OLSRV2_ROUTING, "Trigger routing update");
 }
 
+/**
+ * Freeze all modifications of all OLSRv2 routing table
+ * @param freeze true to freeze tables, false to update them to
+ *   the dijkstra results again.
+ */
+void
+olsrv2_routing_freeze_routes(bool freeze) {
+  if (_freeze_routes == freeze) {
+    return;
+  }
+
+  _freeze_routes = freeze;
+  if (!freeze) {
+    /* make sure we have a current routing table */
+    olsrv2_routing_trigger_update();
+  }
+}
+
 /**
  * @param domain nhdp domain
  * @return routing domain parameters
@@ -227,6 +284,24 @@ olsrv2_routing_get_parameters(struct nhdp_domain *domain) {
   return &_domain_parameter[domain->index];
 }
 
+/**
+ * Mark a domain as changed to trigger a dijkstra run
+ * @param domain NHDP domain, NULL for all domains
+ */
+void
+olsrv2_routing_domain_changed(struct nhdp_domain *domain) {
+  if (domain) {
+    _domain_changed[domain->index] = true;
+
+    olsrv2_routing_trigger_update();
+    return;
+  }
+
+  list_for_each_element(nhdp_domain_get_list(), domain, _node) {
+    olsrv2_routing_domain_changed(domain);
+  }
+}
+
 /**
  * Trigger dijkstra and routing update now
  * @param skip_wait true to ignore rate limitation timer
@@ -236,7 +311,7 @@ olsrv2_routing_force_update(bool skip_wait) {
   struct nhdp_domain *domain;
   bool splitv4, splitv6;
 
-  if (_initiate_shutdown) {
+  if (_initiate_shutdown || _freeze_routes) {
     /* no dijkstra anymore when in shutdown */
     return;
   }
@@ -253,9 +328,22 @@ olsrv2_routing_force_update(bool skip_wait) {
     oonf_timer_stop(&_rate_limit_timer);
   }
 
+  if (_update_ansn) {
+    _ansn++;
+    _update_ansn = false;
+    OONF_DEBUG(LOG_OLSRV2_ROUTING, "Update ANSN to %u", _ansn);
+  }
+
   OONF_DEBUG(LOG_OLSRV2_ROUTING, "Run Dijkstra");
 
   list_for_each_element(nhdp_domain_get_list(), domain, _node) {
+    /* check if dijkstra is necessary */
+    if (!_domain_changed[domain->index]) {
+      /* nothing to do for this domain */
+      continue;
+    }
+    _domain_changed[domain->index] = false;
+
     /* initialize dijkstra specific fields */
     _prepare_routes(domain);
     _prepare_nodes();
@@ -370,6 +458,46 @@ olsrv2_routing_get_filter_list(void) {
   return &_routing_filter_list;
 }
 
+/**
+ * Callback triggered when an MPR-set changed
+ * @param domain NHDP domain that changed
+ */
+static void
+_cb_mpr_update(struct nhdp_domain *domain) {
+  if (!domain) {
+    list_for_each_element(nhdp_domain_get_list(), domain, _node) {
+      _cb_mpr_update(domain);
+    }
+    return;
+  }
+
+  OONF_INFO(LOG_OLSRV2, "MPR update for domain %u", domain->index);
+
+  _update_ansn = true;
+  _domain_changed[domain->index] = true;
+  olsrv2_routing_trigger_update();
+}
+
+/**
+ * Callback triggered when an outgoing metric changed
+ * @param domain NHDP domain that changed
+ */
+static void
+_cb_metric_update(struct nhdp_domain *domain) {
+  if (!domain) {
+    list_for_each_element(nhdp_domain_get_list(), domain, _node) {
+      _cb_metric_update(domain);
+    }
+    return;
+  }
+
+  OONF_INFO(LOG_OLSRV2, "Metric update for domain %u", domain->index);
+
+  _update_ansn = true;
+  _domain_changed[domain->index] = true;
+  olsrv2_routing_trigger_update();
+}
+
 /**
  * Run Dijkstra for a set domain, address family and
  * (non-)source-specific nodes
@@ -453,16 +581,16 @@ _remove_entry(struct olsrv2_routing_entry *entry) {
  * Insert a new entry into the dijkstra working queue
  * @param target pointer to tc target
  * @param neigh next hop through which the target can be reached
- * @param linkcost cost of the last hop of the path towards the target
- * @param pathcost remainder of the cost to the target
+ * @param link_cost cost of the last hop of the path towards the target
+ * @param path_cost remainder of the cost to the target
  * @param distance hopcount to be used for the route to the target
  * @param single_hop true if this is a single-hop route, false otherwise
- * @param last_node address of the last originator before we reached the
+ * @param last_originator address of the last originator before we reached the
  *   destination prefix
  */
 static void
 _insert_into_working_tree(struct olsrv2_tc_target *target,
-    struct nhdp_neighbor *neigh, uint32_t linkcost,
+    struct nhdp_neighbor *neigh, uint32_t link_cost,
     uint32_t path_cost, uint8_t path_hops,
     uint8_t distance, bool single_hop,
     const struct netaddr *last_originator) {
@@ -470,7 +598,7 @@ _insert_into_working_tree(struct olsrv2_tc_target *target,
 #ifdef OONF_LOG_DEBUG_INFO
   struct netaddr_str nbuf1, nbuf2;
 #endif
-  if (linkcost > RFC7181_METRIC_MAX) {
+  if ( link_cost > RFC7181_METRIC_MAX) {
     return;
   }
 
@@ -485,7 +613,7 @@ _insert_into_working_tree(struct olsrv2_tc_target *target,
   }
 
   /* calculate new total pathcost */
-  path_cost += linkcost;
+  path_cost += link_cost;
   path_hops += 1;
 
   if (avl_is_node_added(&node->_node)) {
@@ -597,7 +725,7 @@ _update_routing_entry(struct nhdp_domain *domain,
       netaddr_to_string(&nbuf1, &rtentry->route.p.key.dst),
       netaddr_to_string(&nbuf2, &rtentry->route.p.key.src),
       netaddr_to_string(&nbuf3, &first_hop->originator),
-      domain->ext, pathcost, neighdata->best_link->local_if->os_if_listener.data->name);
+      domain->ext, pathcost, neighdata->best_out_link->local_if->os_if_listener.data->name);
 
   /* remember originator */
   memcpy(&rtentry->originator, dst_originator, sizeof(struct netaddr));
@@ -613,12 +741,12 @@ _update_routing_entry(struct nhdp_domain *domain,
 
   /* copy gateway if necessary */
   if (single_hop
-      && netaddr_cmp(&neighdata->best_link->if_addr,
+      && netaddr_cmp(&neighdata->best_out_link->if_addr,
           &rtentry->route.p.key.dst) == 0) {
     netaddr_invalidate(&rtentry->route.p.gw);
   }
   else {
-    memcpy(&rtentry->route.p.gw, &neighdata->best_link->if_addr,
+    memcpy(&rtentry->route.p.gw, &neighdata->best_out_link->if_addr,
         sizeof(struct netaddr));
   }
 }
@@ -626,8 +754,6 @@ _update_routing_entry(struct nhdp_domain *domain,
 /**
  * Initialize internal fields for dijkstra calculation
  * @param domain nhdp domain
- * @return true if source-specific and non-source specific can
- *   be handled in the same dijkstra run, false otherwise
  */
 static void
 _prepare_routes(struct nhdp_domain *domain) {
@@ -641,9 +767,6 @@ _prepare_routes(struct nhdp_domain *domain) {
 
 /**
  * Initialize internal fields for dijkstra calculation
- * @param domain nhdp domain
- * @return true if source-specific and non-source specific can
- *   be handled in the same dijkstra run, false otherwise
  */
 static void
 _prepare_nodes(void) {
@@ -761,6 +884,8 @@ _add_one_hop_nodes(struct nhdp_domain *domain, int af_family,
 /**
  * Remove item from dijkstra working queue and process it
  * @param domain nhdp domain
+ * @param use_non_ss include non-source-specific nodes into working list
+ * @param use_ss include source-specific nodes into working list
  */
 static void
 _handle_working_queue(struct nhdp_domain *domain,
@@ -1078,18 +1203,9 @@ _cb_trigger_dijkstra(struct oonf_timer_instance *ptr __attribute__((unused))) {
   }
 }
 
-/**
- * Callback triggered when neighbor metrics are updates
- * @param neigh
- */
-static void
-_cb_nhdp_update(struct nhdp_neighbor *neigh __attribute__((unused))) {
-  olsrv2_routing_trigger_update();
-}
-
 /**
  * Callback for kernel route processing results
- * @param route pointer to kernel route
+ * @param route OS route data
  * @param error 0 if no error happened
  */
 static void
