@@ -66,6 +66,11 @@
 
 #include "subsystems/oonf_http.h"
 
+#ifndef O_DIRECTORY
+/* uClibc does not define it */
+#define O_DIRECTORY     00200000        /* must be a directory */
+#endif
+
 /* Definitions */
 #define LOG_HTTP _oonf_http_subsystem.logging
 
@@ -442,7 +447,7 @@ _cb_receive_data(struct oonf_stream_session *session) {
 
 /**
  * Close file transfer descriptor during cleanup
- * @param session
+ * @param session stream session to be cleaned up
  */
 static void
 _cb_cleanup_session(struct oonf_stream_session *session) {
@@ -607,6 +612,9 @@ _create_http_header(struct oonf_stream_session *session,
   /* connection-type */
   abuf_puts(&buf, "Connection: closed\r\n");
 
+  /* allow cross domain access */
+  abuf_puts(&buf, "Access-Control-Allow-Origin: *\r\n");
+
   /* MIME type */
   if (content_type == NULL) {
     content_type = HTTP_CONTENTTYPE_HTML;
