@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #include <errno.h>
 
 #include "common/common_types.h"
@@ -48,9 +52,9 @@
 #include "common/netaddr_acl.h"
 #include "core/oonf_logging.h"
 #include "core/oonf_subsystem.h"
-#include "subsystems/oonf_interface.h"
+#include "subsystems/os_interface.h"
 #include "subsystems/oonf_socket.h"
-#include "subsystems/os_socket.h"
+#include "subsystems/os_fd.h"
 #include "subsystems/oonf_packet_socket.h"
 
 /* Defintions */
@@ -60,27 +64,30 @@
 static int _init(void);
 static void _cleanup(void);
 
-static void _packet_add(struct oonf_packet_socket *pktsocket, int sock,
-    union netaddr_socket *local, struct os_interface_data *interf);
+static void _handle_errno1(struct oonf_packet_socket *pktsocket,
+    union netaddr_socket *remote);
+
+static void _packet_add(struct oonf_packet_socket *pktsocket,
+    union netaddr_socket *local, struct os_interface *os_if);
 static int _apply_managed(struct oonf_packet_managed *managed);
 static int _apply_managed_socketpair(int af_type,
     struct oonf_packet_managed *managed,
-    struct os_interface_data *data, bool *changed,
+    struct os_interface *os_if, bool *changed,
     struct oonf_packet_socket *sock,
     struct oonf_packet_socket *mc_sock, struct netaddr *mc_ip);
 static int _apply_managed_socket(struct oonf_packet_managed *managed,
     struct oonf_packet_socket *stream, const struct netaddr *bindto,
-    int port, uint8_t dscp, int protocol, struct os_interface_data *data);
-static void _cb_packet_event_unicast(int fd, void *data, bool r, bool w);
-static void _cb_packet_event_multicast(int fd, void *data, bool r, bool w);
-static void _cb_packet_event(int fd, void *data, bool r, bool w, bool mc);
-static int _cb_interface_listener(struct oonf_interface_listener *l);
+    int port, uint8_t dscp, int protocol, struct os_interface *os_if);
+static void _cb_packet_event_unicast(struct oonf_socket_entry *);
+static void _cb_packet_event_multicast(struct oonf_socket_entry *);
+static void _cb_packet_event(struct oonf_socket_entry *, bool mc);
+static int _cb_interface_listener(struct os_interface_listener *l);
 
 /* subsystem definition */
 static const char *_dependencies[] = {
-  OONF_INTERFACE_SUBSYSTEM,
+  OONF_OS_INTERFACE_SUBSYSTEM,
   OONF_SOCKET_SUBSYSTEM,
-  OONF_OS_SOCKET_SUBSYSTEM,
+  OONF_OS_FD_SUBSYSTEM,
 };
 
 static struct oonf_subsystem _oonf_packet_socket_subsystem = {
@@ -124,29 +131,27 @@ _cleanup(void) {
  * Add a new packet socket handler
  * @param pktsocket pointer to an initialized packet socket struct
  * @param local pointer local IP address of packet socket
- * @param interf pointer to interface to bind socket on, NULL
+ * @param os_if pointer to interface to bind socket on, NULL
  *   if socket should not be bound to interface
  * @return -1 if an error happened, 0 otherwise
  */
 int
 oonf_packet_add(struct oonf_packet_socket *pktsocket,
-    union netaddr_socket *local, struct os_interface_data *interf) {
-  int s = -1;
-
+    union netaddr_socket *local, struct os_interface *os_if) {
   /* Init socket */
-  s = os_socket_getsocket(local, false, 0, interf, LOG_PACKET);
-  if (s < 0) {
+  if (os_fd_getsocket(
+      &pktsocket->scheduler_entry.fd, local, false, 0, os_if, LOG_PACKET)) {
     return -1;
   }
 
-  _packet_add(pktsocket, s, local, interf);
+  _packet_add(pktsocket, local, os_if);
   return 0;
 }
 
 /**
  * Add a new raw packet socket handler
  * @param pktsocket pointer to an initialized packet socket struct
- * @param int IP protocol number
+ * @param protocol IP protocol number
  * @param local pointer local IP address of packet socket
  * @param interf pointer to interface to bind socket on, NULL
  *   if socket should not be bound to interface
@@ -154,41 +159,45 @@ oonf_packet_add(struct oonf_packet_socket *pktsocket,
  */
 int
 oonf_packet_raw_add(struct oonf_packet_socket *pktsocket, int protocol,
-    union netaddr_socket *local, struct os_interface_data *interf) {
-  int s = -1;
-
+    union netaddr_socket *local, struct os_interface *interf) {
   /* Init socket */
-  s = os_socket_getrawsocket(local, protocol, 0, interf, LOG_PACKET);
-  if (s < 0) {
+  if (os_fd_getrawsocket(
+      &pktsocket->scheduler_entry.fd, local, false, 0, interf, LOG_PACKET)) {
     return -1;
   }
 
-  _packet_add(pktsocket, s, local, interf);
+  _packet_add(pktsocket, local, interf);
   pktsocket->protocol = protocol;
   return 0;
 }
 
 
 static void
-_packet_add(struct oonf_packet_socket *pktsocket, int sock,
-    union netaddr_socket *local, struct os_interface_data *interf) {
-  pktsocket->interface = interf;
-  pktsocket->scheduler_entry.fd = sock;
-  pktsocket->scheduler_entry.process = _cb_packet_event_unicast;
-  pktsocket->scheduler_entry.event_read = true;
-  pktsocket->scheduler_entry.event_write = false;
-  pktsocket->scheduler_entry.data = pktsocket;
+_packet_add(struct oonf_packet_socket *pktsocket,
+    union netaddr_socket *local, struct os_interface *interf) {
+  struct netaddr_str nbuf;
 
-  oonf_socket_add(&pktsocket->scheduler_entry);
+  pktsocket->os_if = interf;
+  pktsocket->scheduler_entry.name = pktsocket->socket_name;
+  pktsocket->scheduler_entry.process = _cb_packet_event_unicast;
 
   abuf_init(&pktsocket->out);
   list_add_tail(&_packet_sockets, &pktsocket->node);
   memcpy(&pktsocket->local_socket, local, sizeof(pktsocket->local_socket));
 
+  /* generate socket name */
+  snprintf(pktsocket->socket_name, sizeof(pktsocket->socket_name),
+      "udp: %s", netaddr_socket_to_string(&nbuf, &pktsocket->local_socket));
+
+  pktsocket->_errno1_measurement_time = oonf_clock_getNow();
+
   if (pktsocket->config.input_buffer_length == 0) {
     pktsocket->config.input_buffer = _input_buffer;
     pktsocket->config.input_buffer_length = sizeof(_input_buffer);
   }
+
+  oonf_socket_add(&pktsocket->scheduler_entry);
+  oonf_socket_set_read(&pktsocket->scheduler_entry, true);
 }
 
 /**
@@ -203,12 +212,10 @@ oonf_packet_remove(struct oonf_packet_socket *pktsocket,
   // TODO: implement non-force behavior for UDP sockets
   if (list_is_node_added(&pktsocket->node)) {
     oonf_socket_remove(&pktsocket->scheduler_entry);
-    os_socket_close(pktsocket->scheduler_entry.fd);
+    os_fd_close(&pktsocket->scheduler_entry.fd);
     abuf_free(&pktsocket->out);
 
     list_remove(&pktsocket->node);
-
-    pktsocket->scheduler_entry.fd = -1;
   }
 }
 
@@ -229,16 +236,21 @@ oonf_packet_send(struct oonf_packet_socket *pktsocket, union netaddr_socket *rem
 
   if (abuf_getlen(&pktsocket->out) == 0) {
     /* no backlog of outgoing packets, try to send directly */
-    result = os_socket_sendto(pktsocket->scheduler_entry.fd, data, length, remote,
+    result = os_fd_sendto(&pktsocket->scheduler_entry.fd, data, length, remote,
         pktsocket->config.dont_route);
     if (result > 0) {
       /* successful */
       OONF_DEBUG(LOG_PACKET, "Sent %d bytes to %s %s",
           result, netaddr_socket_to_string(&buf, remote),
-          pktsocket->interface != NULL ? pktsocket->interface->name : "");
+          pktsocket->os_if != NULL ? pktsocket->os_if->name : "");
+      oonf_socket_register_direct_send(&pktsocket->scheduler_entry);
       return 0;
     }
 
+    if (errno == EPERM) {
+      _handle_errno1(pktsocket, remote);
+      return -1;
+    }
     if (errno != EINTR && errno != EAGAIN && errno != EWOULDBLOCK) {
       OONF_WARN(LOG_PACKET, "Cannot send UDP packet to %s: %s (%d)",
           netaddr_socket_to_string(&buf, remote), strerror(errno), errno);
@@ -271,7 +283,7 @@ oonf_packet_add_managed(struct oonf_packet_managed *managed) {
     managed->config.input_buffer_length = sizeof(_input_buffer);
   }
 
-  managed->_if_listener.process = _cb_interface_listener;
+  managed->_if_listener.if_changed = _cb_interface_listener;
   managed->_if_listener.name = managed->_managed_config.interface;
   managed->_if_listener.mesh = managed->_managed_config.mesh;
 }
@@ -288,7 +300,7 @@ oonf_packet_remove_managed(struct oonf_packet_managed *managed, bool forced) {
   oonf_packet_remove(&managed->multicast_v4, forced);
   oonf_packet_remove(&managed->multicast_v6, forced);
 
-  oonf_interface_remove_listener(&managed->_if_listener);
+  os_interface_remove(&managed->_if_listener);
   oonf_packet_free_managed_config(&managed->_managed_config);
 }
 
@@ -301,7 +313,7 @@ oonf_packet_remove_managed(struct oonf_packet_managed *managed, bool forced) {
  */
 int
 oonf_packet_apply_managed(struct oonf_packet_managed *managed,
-    struct oonf_packet_managed_config *config) {
+    const struct oonf_packet_managed_config *config) {
   bool if_changed;
   int result;
 
@@ -313,21 +325,21 @@ oonf_packet_apply_managed(struct oonf_packet_managed *managed,
   /* handle change in interface listener */
   if (if_changed) {
     /* interface changed, remove old listener if necessary */
-    oonf_interface_remove_listener(&managed->_if_listener);
+    os_interface_remove(&managed->_if_listener);
 
     /* create new interface listener */
     managed->_if_listener.mesh = managed->_managed_config.mesh;
-    oonf_interface_add_listener(&managed->_if_listener);
+    os_interface_add(&managed->_if_listener);
   }
 
   OONF_DEBUG(LOG_PACKET, "Apply changes for managed socket (if %s) with port %d/%d",
-      config->interface == NULL || config->interface[0] == 0 ? "any" : config->interface,
+      config->interface[0] == 0 ? "any" : config->interface,
       config->port, config->multicast_port);
 
   result = _apply_managed(managed);
   if (result) {
     /* did not work, trigger interface handler to try later again */
-    oonf_interface_trigger_handler(&managed->_if_listener);
+    os_interface_trigger_handler(&managed->_if_listener);
   }
   return result;
 }
@@ -418,15 +430,18 @@ oonf_packet_managed_is_active(
  */
 void
 oonf_packet_copy_managed_config(struct oonf_packet_managed_config *dst,
-    struct oonf_packet_managed_config *src) {
+    const struct oonf_packet_managed_config *src) {
   oonf_packet_free_managed_config(dst);
 
+  /* careful, we are doing a shallow copy, so both ACLs are BAD after this */
   memcpy(dst, src, sizeof(*dst));
 
+  /* fix it to make sure we don't use-after-free or double-free */
   memset(&dst->acl, 0, sizeof(dst->acl));
-  netaddr_acl_copy(&dst->acl, &src->acl);
-
   memset(&dst->bindto, 0, sizeof(dst->bindto));
+
+  /* now do a deep copy of the ACLs */
+  netaddr_acl_copy(&dst->acl, &src->acl);
   netaddr_acl_copy(&dst->bindto, &src->bindto);
 }
 
@@ -440,30 +455,92 @@ oonf_packet_free_managed_config(struct oonf_packet_managed_config *config) {
   netaddr_acl_remove(&config->bindto);
 }
 
+/**
+ * Handle rate limitation of errno==1 warnings
+ * @param pktsocket packet socket the error happened
+ * @param remote target of UDP packet
+ */
+static void
+_handle_errno1(struct oonf_packet_socket *pktsocket,
+    union netaddr_socket *remote) {
+  int64_t interval;
+  uint32_t count, normalized;
+  struct netaddr_str buf;
+  const char *ifname = "-";
+  bool triggered;
+
+  /* we do a rate limitation for EPERM */
+  interval = -oonf_clock_get_relative(pktsocket->_errno1_measurement_time);
+  count = pktsocket->_errno1_count;
+  normalized = count;
+  if (interval >= 60000) {
+    normalized /= (interval / 60000);
+  }
+
+  /* increase error counter */
+  pktsocket->_errno1_count++;
+
+  triggered = false;
+  if (interval >= 60000 && pktsocket->_errno1_suppression && normalized < 5) {
+    /* switch suppression on */
+    pktsocket->_errno1_suppression = false;
+  }
+  else if (!pktsocket->_errno1_suppression && normalized > 10) {
+    /* switch suppression off */
+    pktsocket->_errno1_suppression = true;
+    count = 0;
+    triggered = true;
+  }
+
+  if (interval >= 60000 || triggered) {
+    /* start new measurement interval */
+    pktsocket->_errno1_measurement_time = oonf_clock_getNow();
+    pktsocket->_errno1_count = 1;
+  }
+
+  if (pktsocket->os_if) {
+    /* remember interface name */
+    ifname = pktsocket->os_if->name;
+  }
+
+  if (pktsocket->_errno1_suppression && count != 0) {
+    if (pktsocket->_errno1_count > 1) {
+      /* suppress warning */
+      return;
+    }
+
+    OONF_WARN(LOG_PACKET, "Cannot send UDP packet to %s (%s): %s (%d) (%d similar errors suppressed)",
+        netaddr_socket_to_string(&buf, remote), ifname, strerror(errno), errno, count);
+  }
+  else {
+    OONF_WARN(LOG_PACKET, "Cannot send UDP packet to %s (%s): %s (%d)",
+        netaddr_socket_to_string(&buf, remote), ifname, strerror(errno), errno);
+  }
+}
+
 /**
  * Apply a new configuration to all attached sockets
  * @param managed pointer to managed socket
- * @param config pointer to configuration
  * @return -1 if an error happened, 0 otherwise
  */
 static int
 _apply_managed(struct oonf_packet_managed *managed) {
-  struct os_interface_data *data = NULL;
+  struct os_interface *os_if = NULL;
   bool changed = false;
   int result = 0;
 
   /* get interface */
-  if (managed->_if_listener.interface) {
-    data = &managed->_if_listener.interface->data;
+  if (managed->_if_listener.name) {
+    os_if = managed->_if_listener.data;
   }
 
-  if (_apply_managed_socketpair(AF_INET, managed, data, &changed,
+  if (_apply_managed_socketpair(AF_INET, managed, os_if, &changed,
       &managed->socket_v4, &managed->multicast_v4,
       &managed->_managed_config.multicast_v4)) {
     result = -1;
   }
 
-  if (_apply_managed_socketpair(AF_INET6, managed, data, &changed,
+  if (_apply_managed_socketpair(AF_INET6, managed, os_if, &changed,
       &managed->socket_v6, &managed->multicast_v6,
       &managed->_managed_config.multicast_v6)) {
     result = -1;
@@ -477,17 +554,17 @@ _apply_managed(struct oonf_packet_managed *managed) {
 
 /**
  * Apply a new configuration to an unicast/multicast socket pair
- * @param managed pointer to managed socket
- * @param data pointer to interface to bind sockets, NULL if unbound socket
- * @param sock pointer to unicast packet socket
- * @param bind_ip address to bind unicast socket to
+ * @param managed managed socket
+ * @param os_if OS interface to bind sockets, NULL if unbound socket
+ * @param changed buffer for boolean, will be set to true if sockets changed
+ * @param sock unicast packet socket
  * @param mc_sock pointer to multicast packet socket
  * @param mc_ip multicast address
- * @return
+ * @return 0 if okay, -1 if an error happened
  */
 static int
 _apply_managed_socketpair(int af_type, struct oonf_packet_managed *managed,
-    struct os_interface_data *data, bool *changed,
+    struct os_interface *os_if, bool *changed,
     struct oonf_packet_socket *sock,
     struct oonf_packet_socket *mc_sock, struct netaddr *mc_ip) {
   struct netaddr_acl *bind_ip_acl;
@@ -505,16 +582,21 @@ _apply_managed_socketpair(int af_type, struct oonf_packet_managed *managed,
   }
 
   /* Get address the unicast socket should bind on */
-  if (data != NULL && !data->up) {
+  if (os_if != NULL && !os_if->flags.up) {
     bind_ip = NULL;
   }
-  else if (data != NULL && netaddr_get_address_family(data->linklocal_v6_ptr) == af_type &&
-      netaddr_acl_check_accept(bind_ip_acl, data->linklocal_v6_ptr)) {
+  else if (os_if != NULL && netaddr_get_address_family(os_if->if_linklocal_v6) == af_type &&
+      netaddr_acl_check_accept(bind_ip_acl, os_if->if_linklocal_v6)) {
 
-    bind_ip = data->linklocal_v6_ptr;
+    bind_ip = os_if->if_linklocal_v6;
+  }
+  else if (os_if != NULL && netaddr_get_address_family(os_if->if_linklocal_v4) == af_type &&
+      netaddr_acl_check_accept(bind_ip_acl, os_if->if_linklocal_v4)) {
+
+    bind_ip = os_if->if_linklocal_v4;
   }
   else {
-    bind_ip = oonf_interface_get_bindaddress(af_type, bind_ip_acl, data);
+    bind_ip = os_interface_get_bindaddress(af_type, bind_ip_acl, os_if);
   }
   if (!bind_ip) {
     oonf_packet_remove(sock, false);
@@ -523,7 +605,7 @@ _apply_managed_socketpair(int af_type, struct oonf_packet_managed *managed,
   }
 
   /* handle loopback interface */
-  if (data != NULL && data->loopback
+  if (os_if != NULL && os_if->flags.loopback
       && netaddr_get_address_family(mc_ip) != AF_UNSPEC) {
     memcpy(mc_ip, bind_ip, sizeof(*mc_ip));
   }
@@ -538,14 +620,15 @@ _apply_managed_socketpair(int af_type, struct oonf_packet_managed *managed,
       managed, sock, bind_ip, managed->_managed_config.port,
       managed->_managed_config.dscp,
       managed->_managed_config.rawip ? managed->_managed_config.protocol : 0,
-      data);
+      os_if);
   if (sockstate == 0) {
     /* settings really changed */
     *changed = true;
 
-    if (real_multicast && data != NULL && data->up) {
-      os_socket_join_mcast_send(sock->scheduler_entry.fd,
-          mc_ip, data, managed->_managed_config.loop_multicast, LOG_PACKET);
+    if (real_multicast && os_if != NULL && os_if->flags.up) {
+      os_fd_join_mcast_send(&sock->scheduler_entry.fd, mc_ip, os_if,
+          managed->_managed_config.loop_multicast,
+          managed->_managed_config.ttl_multicast, LOG_PACKET);
     }
   }
   else if (sockstate < 0) {
@@ -559,7 +642,7 @@ _apply_managed_socketpair(int af_type, struct oonf_packet_managed *managed,
     sockstate = _apply_managed_socket(
         managed, mc_sock, mc_ip, mc_port, managed->_managed_config.dscp,
         managed->_managed_config.rawip ? managed->_managed_config.protocol : 0,
-        data);
+        os_if);
     if (sockstate == 0) {
       /* settings really changed */
       *changed = true;
@@ -567,8 +650,8 @@ _apply_managed_socketpair(int af_type, struct oonf_packet_managed *managed,
       mc_sock->scheduler_entry.process = _cb_packet_event_multicast;
 
       /* join multicast group */
-      os_socket_join_mcast_recv(mc_sock->scheduler_entry.fd,
-          mc_ip, data, LOG_PACKET);
+      os_fd_join_mcast_recv(&mc_sock->scheduler_entry.fd,
+          mc_ip, os_if, LOG_PACKET);
     }
     else if (sockstate < 0) {
       /* error */
@@ -584,15 +667,15 @@ _apply_managed_socketpair(int af_type, struct oonf_packet_managed *managed,
      * oonf_packet_send_managed_multicast()
      */
     netaddr_socket_init(&mc_sock->local_socket, mc_ip, mc_port,
-        data == NULL ? 0 : data->index);
+        os_if == NULL ? 0 : os_if->index);
   }
   return result;
 }
 
 /**
  * Apply new configuration to a managed stream socket
- * @param managed pointer to managed stream
- * @param stream pointer to TCP stream to configure
+ * @param managed managed packet socket (pair)
+ * @param packet packet socket
  * @param bindto local address to bind socket to
  *   set to AF_UNSPEC for simple reinitialization
  * @param port local port number
@@ -606,7 +689,7 @@ static int
 _apply_managed_socket(struct oonf_packet_managed *managed,
     struct oonf_packet_socket *packet,
     const struct netaddr *bindto, int port, uint8_t dscp,
-    int protocol, struct os_interface_data *data) {
+    int protocol, struct os_interface *data) {
   union netaddr_socket sock;
   struct netaddr_str buf;
 
@@ -619,7 +702,7 @@ _apply_managed_socket(struct oonf_packet_managed *managed,
   }
 
   if (list_is_node_added(&packet->node)) {
-    if (data == packet->interface
+    if (data == packet->os_if
         && memcmp(&sock, &packet->local_socket, sizeof(sock)) == 0
         && protocol == packet->protocol) {
       /* nothing changed */
@@ -627,7 +710,7 @@ _apply_managed_socket(struct oonf_packet_managed *managed,
     }
   }
   else {
-    if (data != NULL && !data->up) {
+    if (data != NULL && !data->flags.up) {
       /* nothing changed */
       return 1;
     }
@@ -636,7 +719,7 @@ _apply_managed_socket(struct oonf_packet_managed *managed,
   /* remove old socket */
   oonf_packet_remove(packet, true);
 
-  if (data != NULL && !data->up) {
+  if (data != NULL && !data->flags.up) {
     OONF_DEBUG(LOG_PACKET, "Interface %s of socket is down",
         data->name);
     return 0;
@@ -660,14 +743,14 @@ _apply_managed_socket(struct oonf_packet_managed *managed,
     }
   }
 
-  if (os_socket_set_dscp(packet->scheduler_entry.fd, dscp,
+  if (os_fd_set_dscp(&packet->scheduler_entry.fd, dscp,
       netaddr_get_address_family(bindto) == AF_INET6)) {
     OONF_WARN(LOG_PACKET, "Could not set DSCP value for socket: %s (%d)",
         strerror(errno), errno);
     oonf_packet_remove(packet, true);
     return -1;
   }
-  packet->interface = data;
+  packet->os_if = data;
 
   OONF_DEBUG(LOG_PACKET, "Opened new socket and bound it to %s (if %s)",
       netaddr_to_string(&buf, bindto),
@@ -677,40 +760,33 @@ _apply_managed_socket(struct oonf_packet_managed *managed,
 
 /**
  * callback for unicast events in socket scheduler
- * @param fd
- * @param data
- * @param event_read
- * @param event_write
+ * @param entry socket entry that fired event
  */
 static void
-_cb_packet_event_unicast(int fd, void *data, bool event_read, bool event_write) {
-  _cb_packet_event(fd, data, event_read, event_write, false);
+_cb_packet_event_unicast(struct oonf_socket_entry *entry) {
+  _cb_packet_event(entry, false);
 }
 
 /**
  * callback for multicast events in socket scheduler
- * @param fd
- * @param data
- * @param event_read
- * @param event_write
+ * @param entry socket entry that fired event
  */
 static void
-_cb_packet_event_multicast(int fd, void *data, bool event_read, bool event_write) {
-  _cb_packet_event(fd, data, event_read, event_write, true);
+_cb_packet_event_multicast(struct oonf_socket_entry *entry) {
+  _cb_packet_event(entry, true);
 }
 
 /**
  * Callback to handle data from the olsr socket scheduler
- * @param fd filedescriptor to read data from
- * @param data custom data pointer
- * @param event_read true if read-event is incoming
- * @param event_write true if write-event is incoming
+ * @param entry socket entry that fired event
+ * @param multicast true if the multicast socket fired the event,
+ *   false otherwise
  */
 static void
-_cb_packet_event(int fd, void *data, bool event_read, bool event_write,
+_cb_packet_event(struct oonf_socket_entry *entry,
     bool multicast __attribute__((unused))) {
-  struct oonf_packet_socket *pktsocket = data;
-  union netaddr_socket *skt, sock;
+  struct oonf_packet_socket *pktsocket;
+  union netaddr_socket sock;
   uint16_t length;
   char *pkt;
   ssize_t result;
@@ -718,13 +794,17 @@ _cb_packet_event(int fd, void *data, bool event_read, bool event_write,
 
 #ifdef OONF_LOG_DEBUG_INFO
   const char *interf = "";
+#endif
 
-  if (pktsocket->interface) {
-    interf = pktsocket->interface->name;
+  pktsocket = container_of(entry, typeof(*pktsocket), scheduler_entry);
+
+#ifdef OONF_LOG_DEBUG_INFO
+  if (pktsocket->os_if) {
+    interf = pktsocket->os_if->name;
   }
 #endif
 
-  if (event_read) {
+  if (oonf_socket_is_read(entry)) {
     uint8_t *buf;
 
     /* clear recvfrom memory */
@@ -733,12 +813,13 @@ _cb_packet_event(int fd, void *data, bool event_read, bool event_write,
     /* handle incoming data */
     buf = pktsocket->config.input_buffer;
 
-    result = os_socket_recvfrom(fd, buf, pktsocket->config.input_buffer_length-1, &sock,
-        pktsocket->interface);
+    result = os_fd_recvfrom(&entry->fd,
+        buf, pktsocket->config.input_buffer_length-1, &sock,
+        pktsocket->os_if);
     if (result > 0 && pktsocket->config.receive_data != NULL) {
       /* handle raw socket */
       if (pktsocket->protocol) {
-        buf = os_socket_skip_rawsocket_prefix(buf, &result, pktsocket->local_socket.std.sa_family);
+        buf = os_fd_skip_rawsocket_prefix(buf, &result, pktsocket->local_socket.std.sa_family);
         if (!buf) {
           OONF_WARN(LOG_PACKET, "Error while skipping IP header for socket %s:",
               netaddr_socket_to_string(&netbuf, &pktsocket->local_socket));
@@ -760,39 +841,38 @@ _cb_packet_event(int fd, void *data, bool event_read, bool event_write,
     }
   }
 
-  if (event_write && abuf_getlen(&pktsocket->out) > 0) {
+  if (oonf_socket_is_write(entry) && abuf_getlen(&pktsocket->out) > 0) {
     /* handle outgoing data */
+    pkt = abuf_getptr(&pktsocket->out);
 
-    /* pointer to remote socket */
-    skt = data;
-
-    /* data area */
-    pkt = data;
-    pkt += sizeof(*skt);
+    /* copy remote socket */
+    memcpy(&sock, pkt, sizeof(sock));
+    pkt += sizeof(sock);
 
+    /* copy length */
     memcpy(&length, pkt, 2);
     pkt += 2;
 
     /* try to send packet */
-    result = os_socket_sendto(fd, data, length, skt, pktsocket->config.dont_route);
+    result = os_fd_sendto(&entry->fd, pkt, length, &sock, pktsocket->config.dont_route);
     if (result < 0 && (errno == EINTR || errno == EAGAIN || errno == EWOULDBLOCK)) {
       /* try again later */
       OONF_DEBUG(LOG_PACKET, "Sending to %s %s could block, try again later",
-          netaddr_socket_to_string(&netbuf, skt), interf);
+          netaddr_socket_to_string(&netbuf, &sock), interf);
       return;
     }
 
     if (result < 0) {
       /* display error message */
       OONF_WARN(LOG_PACKET, "Cannot send UDP packet to %s: %s (%d)",
-          netaddr_socket_to_string(&netbuf, skt), strerror(errno), errno);
+          netaddr_socket_to_string(&netbuf, &sock), strerror(errno), errno);
     }
     else {
       OONF_DEBUG(LOG_PACKET, "Sent %"PRINTF_SSIZE_T_SPECIFIER" bytes to %s %s",
-          result, netaddr_socket_to_string(&netbuf, skt), interf);
+          result, netaddr_socket_to_string(&netbuf, &sock), interf);
     }
     /* remove data from outgoing buffer (both for success and for final error */
-    abuf_pull(&pktsocket->out, sizeof(*skt) + 2 + length);
+    abuf_pull(&pktsocket->out, sizeof(sock) + 2 + length);
   }
 
   if (abuf_getlen(&pktsocket->out) == 0) {
@@ -803,11 +883,11 @@ _cb_packet_event(int fd, void *data, bool event_read, bool event_write,
 
 /**
  * Callbacks for events on the interface
- * @param l
+ * @param l OS interface listener
  * @return -1 if an error happened, 0 otherwise
  */
 static int
-_cb_interface_listener(struct oonf_interface_listener *l) {
+_cb_interface_listener(struct os_interface_listener *l) {
   struct oonf_packet_managed *managed;
   int result;
 
