@@ -50,7 +50,7 @@
 #include "rfc5444_context.h"
 #include "rfc5444_api_config.h"
 
-static size_t _calc_tlv_size(bool exttype, size_t length);
+static size_t _calc_tlv_size(bool has_exttype, size_t length);
 static void _write_tlv(uint8_t *ptr, uint8_t type, uint8_t exttype,
     uint8_t idx1, uint8_t idx2, const void *value, size_t length);
 
@@ -162,9 +162,9 @@ _rfc5444_tlv_writer_set(struct rfc5444_tlv_writer_data *data,
  * @param length number of bytes of tlv value, 0 if no value
  * @return number of bytes for TLV including header
  */
-static size_t _calc_tlv_size(bool exttype, size_t length) {
+static size_t _calc_tlv_size(bool has_exttype, size_t length) {
   size_t s = 2;
-  if (exttype) {
+  if (has_exttype) {
     s++;
   }
 
@@ -183,7 +183,7 @@ static size_t _calc_tlv_size(bool exttype, size_t length) {
  * This function does NOT do a length check before writing
  * @param ptr pointer to buffer
  * @param type tlv type
- * @param has_exttype extended tlv type, 0 if no extended type
+ * @param exttype extended tlv type, 0 if no extended type
  * @param idx1 lower index of tlv in address block, 0 if no address tlv
  * @param idx2 upper index of tlv in address block, 255 if no address tlv
  * @param value pointer to tlv value, NULL if no value
