@@ -107,6 +107,7 @@ rfc5444_writer_init(struct rfc5444_writer *writer) {
 
   avl_init(&writer->_msgcreators, avl_comp_uint8, false);
   avl_init(&writer->_processors, avl_comp_int32, true);
+  avl_init(&writer->_forwarding_processors, avl_comp_int32, true);
 
 #if WRITER_STATE_MACHINE == true
   writer->_state = RFC5444_WRITER_NONE;
@@ -275,8 +276,6 @@ rfc5444_writer_add_address(struct rfc5444_writer *writer __attribute__ ((unused)
  * This function must NOT be called from the rfc5444 writer callbacks.
  *
  * @param writer pointer to writer context
- * @param msgtype messagetype for this tlv, -1 to registere a generic
- *   (not message specific) address TLV type
  * @param type pointer to tlvtype structure, type and exttype must be already initialized
  * @param msgtype message type this TLV belongs too, -1 for address TLV
  * @return 0 if addresstlvtype was registered, -1 otherwise
@@ -477,6 +476,31 @@ rfc5444_writer_unregister_postprocessor(struct rfc5444_writer *writer,
   }
 }
 
+/**
+ * Registers a new forward handler
+ * @param writer rfc5444 writer
+ * @param forward rfc5444 forward handler
+ */
+void
+rfc5444_writer_register_forward_handler(struct rfc5444_writer *writer,
+    struct rfc5444_writer_forward_handler *forward) {
+  forward->_node.key = &forward->priority;
+  avl_insert(&writer->_forwarding_processors, &forward->_node);
+}
+
+/**
+ * Unregisters a forward handler
+ * @param writer rfc5444 writer
+ * @param forward rfc5444 forward handler
+ */
+void
+rfc5444_writer_unregister_forward_handler(struct rfc5444_writer *writer,
+    struct rfc5444_writer_forward_handler *forward) {
+  if (avl_is_node_added(&forward->_node)) {
+    avl_remove(&writer->_forwarding_processors, &forward->_node);
+  }
+}
+
 /**
  * Registers a packet handler for the writer
  * This function must not be called outside the message_addresses callback.
@@ -611,6 +635,8 @@ _register_addrtlvtype(struct rfc5444_writer *writer,
   tlvtype->_creator = msg;
   tlvtype->_full_type = _get_fulltype(tlvtype->type, tlvtype->exttype);
 
+  assert(!list_is_node_added(&tlvtype->_tlvtype_node));
+
   if (msg) {
     /* add to message creator list */
     list_add_tail(&msg->_msgspecific_tlvtype_head, &tlvtype->_tlvtype_node);
@@ -671,7 +697,7 @@ _rfc5444_writer_free_addresses(struct rfc5444_writer *writer, struct rfc5444_wri
 /**
  * Free message object if not in use anymore
  * @param writer pointer to writer context
- * @param _msg pointer to message object
+ * @param msg pointer to message object
  */
 static void
 _lazy_free_message(struct rfc5444_writer *writer, struct rfc5444_writer_message *msg) {
